@@ -6,7 +6,7 @@
  * PLEASE READ THESE TERMS BEFORE DISTRIBUTING.                     *
  *                                                                  *
  * THE Ogg123 SOURCE CODE IS (C) COPYRIGHT 2000-2001                *
- * by Kenneth C. Arnold <ogg@arnoldnet.net> AND OTHER CONTRIBUTORS  *
+ * by Stan Seibert <volsung@xiph.org> AND OTHER CONTRIBUTORS        *
  * http://www.xiph.org/                                             *
  *                                                                  *
  ********************************************************************
@@ -15,16 +15,29 @@
 
  ********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <pthread.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
 #include "status.h"
 #include "i18n.h"
 
 char temp_buffer[200];
 int last_line_len = 0;
 int max_verbosity = 0;
+int exit_status = EXIT_SUCCESS;
 
 pthread_mutex_t output_lock = PTHREAD_MUTEX_INITIALIZER;
 
@@ -308,8 +321,12 @@ void stat_format_cleanup (stat_format_t *stats)
 }
 
 
-void status_set_verbosity (int verbosity)
+void status_init (int verbosity)
 {
+#if defined(HAVE_FCNTL) && defined(HAVE_UNISTD_H)
+  fcntl (STDERR_FILENO, F_SETFL, O_NONBLOCK);
+#endif
+
   max_verbosity = verbosity;
 }
 
@@ -449,6 +466,8 @@ void status_error (const char *fmt, ...)
   pthread_mutex_unlock(&output_lock);
 
   pthread_cleanup_pop(0);
+
+  exit_status = EXIT_FAILURE;
 }
 
 
@@ -462,4 +481,6 @@ void vstatus_error (const char *fmt, va_list ap)
   pthread_mutex_unlock(&output_lock);
 
   pthread_cleanup_pop(0);
+
+  exit_status = EXIT_FAILURE;
 }
